<?php
namespace tests\Util;

use Util\Period;
use Moment\Moment;

class PeriodTest extends \PHPUnit_Framework_TestCase {
    /** @var Period */
    protected $periodService;
    /** @var Moment */
    protected $start;
    /** @var Moment */
    protected $end;

    public function setUp() {
        $this->periodService = new Period();
        $this->start = new Moment('now', "America/Sao_Paulo");
        $this->end = new Moment('now', "America/Sao_Paulo");
    }

    public function testDecodeToday() {
        $this->assertPeriod("today");
        $this->assertPeriod("this-day");
    }

    public function testDecodeTomorrow() {
        $this->start->addDays();
        $this->end->addDays();

        $this->assertPeriod("tomorrow");
        $this->assertPeriod("next-day");
    }

    public function testDecodeYesterday() {
        $this->start->subtractDays();
        $this->end->subtractDays();

        $this->assertPeriod("yesterday");
        $this->assertPeriod("last-day");
    }

    public function testDecodeNextNDays() {
        $this->start->startOf('day')->addDays();
        $this->end->endOf('day')->addDays(2);
        $this->assertPeriod("next-2-day");
    }

    public function testDecodeNextNDaysInclusive() {
        $this->start->startOf('day');
        $this->end->endOf('day')->addDays(2);
        $this->assertPeriod("next-3-day-inclusive");
    }

    public function testDecodeLastNDays() {
        $this->start->startOf('day')->subtractDays(5);
        $this->end->endOf('day')->subtractDays();
        $this->assertPeriod("last-5-day");
    }

    public function testDecodeLastNDaysInclusive() {
        $this->start->startOf('day')->subtractDays(5);
        $this->end->endOf('day');
        $this->assertPeriod("last-6-day-inclusive");
    }

    public function testDecodeDayAlias() {
        $this->start->startOf('day')->addDays();
        $this->end->endOf('day')->addDays(2);
        $this->assertPeriod("next-2-days");
    }

    public function testDecodeThisWeek() {
        $this->start->startOf('week')->subtractDays();
        $this->end->endOf('week')->subtractDays();
        $this->assertPeriod("this-week");
    }

    public function testDecodeNextWeek() {
        $this->start->startOf('week')->subtractDays()->addWeeks();
        $this->end->endOf('week')->subtractDays()->addWeeks();
        $this->assertPeriod("next-week");
    }

    public function testDecodeLastWeek() {
        $this->start->startOf('week')->subtractDays()->subtractWeeks();
        $this->end->endOf('week')->subtractDays()->subtractWeeks();
        $this->assertPeriod("last-week");
    }

    public function testDecodeNextNWeeks() {
        $this->start->addWeeks()->startOf('week')->subtractDays();
        $this->end->addWeeks(2)->endOf('week')->subtractDays();
        $this->assertPeriod("next-2-week");
    }

    public function testDecodeNextNWeeksInclusive() {
        $this->start->startOf('week')->subtractDays();
        $this->end->addWeeks(2)->endOf('week')->subtractDays();
        $this->assertPeriod("next-3-week-inclusive");
    }

    public function testDecodeLastNWeeks() {
        $this->start->subtractWeeks(5)->startOf('week')->subtractDays();
        $this->end->subtractWeeks()->endOf('week')->subtractDays();
        $this->assertPeriod("last-5-week");
    }

    public function testDecodeLastNWeeksInclusive() {
        $this->start->subtractWeeks(4)->startOf('week')->subtractDays();
        $this->end->endOf('week')->subtractDays();
        $this->assertPeriod("last-5-week-inclusive");
    }

    public function testDecodeWeekAlias() {
        $this->start->addWeeks()->startOf('week')->subtractDays();
        $this->end->addWeeks(2)->endOf('week')->subtractDays();
        $this->assertPeriod("next-2-weeks");
    }

    public function testDecodeThisMonth() {
        $this->start->startOf('month');
        $this->end->endOf('month');
        $this->assertPeriod("this-month");
    }

    public function testDecodeNextMonth() {
        $this->start->addMonths()->startOf('month');
        $this->end->addMonths()->endOf('month');
        $this->assertPeriod("next-month");
    }

    public function testDecodeLastMonth() {
        $this->start->subtractMonths()->startOf('month');
        $this->end->subtractMonths()->endOf('month');
        $this->assertPeriod("last-month");
    }

    public function testDecodeNextNMonths() {
        $this->start->addMonths()->startOf('month');
        $this->end->addMonths(3)->endOf('month');
        $this->assertPeriod("next-3-month");
    }

    public function testDecodeNextNMonthsInclusive() {
        $this->start->startOf('month');
        $this->end->addMonths(2)->endOf('month');
        $this->assertPeriod("next-3-month-inclusive");
    }

    public function testDecodeLastNMonths() {
        $this->start->subtractMonths(5)->startOf('month');
        $this->end->subtractMonths()->endOf('month');
        $this->assertPeriod("last-5-month");
    }

    public function testDecodeLastNMonthsInclusive() {
        $this->start->subtractMonths(4)->startOf('month');
        $this->end->endOf('month');
        $this->assertPeriod("last-5-month-inclusive");
    }

    public function testDecodeMonthAlias() {
        $this->start->addMonths()->startOf('month');
        $this->end->addMonths(3)->endOf('month');
        $this->assertPeriod("next-3-months");
    }

    public function testDecodeThisYear() {
        $this->start->startOf('year');
        $this->end->endOf('year');
        $this->assertPeriod("this-year");
    }

    public function testDecodeNextYear() {
        $this->start->addYears()->startOf('year');
        $this->end->addYears()->endOf('year');
        $this->assertPeriod("next-year");
    }

    public function testDecodeLastYear() {
        $this->start->subtractYears()->startOf('year');
        $this->end->subtractYears()->endOf('year');
        $this->assertPeriod("last-year");
    }

    public function testDecodeNextNYears() {
        $this->start->addYears()->startOf('year');
        $this->end->addYears(2)->endOf('year');
        $this->assertPeriod("next-2-year");
    }

    public function testDecodeNextNYearsInclusive() {
        $this->start->startOf('year');
        $this->end->addYears(2)->endOf('year');
        $this->assertPeriod("next-3-year-inclusive");
    }

    public function testDecodeLastNYears() {
        $this->start->subtractYears(5)->startOf('year');
        $this->end->subtractYears()->endOf('year');
        $this->assertPeriod("last-5-year");
    }

    public function testDecodeLastNYearsInclusive() {
        $this->start->subtractYears(4)->startOf('year');
        $this->end->endOf('year');
        $this->assertPeriod("last-5-year-inclusive");
    }

    public function testDecodeYearAlias() {
        $this->start->addYears()->startOf('year');
        $this->end->addYears(2)->endOf('year');
        $this->assertPeriod("next-2-years");
    }

    public function testDecodeWithInvalidPeriod() {
        $this->setExpectedException("Exception", "Invalid period format. Missing informations.");
        $this->assertPeriod("year");
    }

    public function testDecodeWithInvalidprefix() {
        $this->setExpectedException("Exception", "Invalid period format. Invalid prefix: something.");
        $this->assertPeriod("something-year");
    }

    public function testDecodeWithInvalidSuffix() {
        $this->setExpectedException("Exception", "Invalid period format. Invalid suffix: something.");
        $this->assertPeriod("next-3-year-something");
    }

    public function testDecodeWithInvalidUnit() {
        $this->setExpectedException("Exception", "Invalid period format. Invalid unit: something.");
        $this->assertPeriod("next-something");
    }

    protected function assertPeriod($period){
        list($start, $end) = $this->periodService->decodeFormatted($period);
        $this->assertEquals($this->start->format('d/m/Y'), $start);
        $this->assertEquals($this->end->format('d/m/Y'), $end);
    }

}